*! NJC 1.1.0 5 Aug 2007
*! NJC 1.0.0 26 Feb 2007
program hsmode, rclass byable(recall)
	version 9
	syntax varlist(numeric) [if] [in] [, ///
	Format(str) BY(varname) Generate(str) ALLobs MISSing ]

	quietly { 
		// what observations to use 
		if "`allobs'" != "" marksample touse, novarlist 
		else marksample touse
		
		if "`by'" != "" & "`missing'" == "" markout `touse' `by', strok 
		
		count if `touse' 
		if r(N) == 0 error 2000

		// variable(s) or group(s) 
		local nv : word count `varlist'

		// generate() option 
		if "`generate'" != "" { 
			if `nv' > 1 { 
				di as err ///
				"generate() cannot be combined with `nv' variables"
				exit 198 
			}

			if _by() { 
				di as err ///
				"generate() cannot be combined with by: : use by() option"
				exit 198 
			}
		
			confirm new var `generate' 
			generate double `generate' = .                  
		}

		// by option 
		if "`by'" != "" { 
			if `nv' > 1 { 
				di as err ///
				"by() cannot be combined with `nv' variables"
				exit 198 
			}

			tempvar group 
			egen `group' = group(`by') if `touse', label  
			su `group', meanonly 
			local nv = r(max)
		} 
		else tokenize `varlist' 

		if "`by'`allobs'" == "" {
			local header "(n = `r(N)')"
		}	
		else if "`by'" != "" { 
			local header : var label `by' 
			if `"`header'"' == "" local header "`by'" 
		}
		else local header "variable" 
			
		// display preparation and initiation 
		tempvar which result thisuse  
		tempname whichlabel 
		gen `which' = _n 
		label var `which' `"`header'"' 
		gen `result' = "" 
		label var `result' "mode"
		
		if "`by'`allobs'" != "" { 
			tempvar n 
			gen `n' = "" 
			label var `n' "n" 
		}

		gen byte `thisuse' = . 
		if "`format'" == "" local format "%9.0g" 

		// loop over variables or groups                        
		forval i = 1/`nv' { 
			if "`by'" != "" { 
				local lbl : label (`group') `i' 
				replace `thisuse' = `touse' & `group' == `i' 
			} 
			else { 
				local lbl : var label ``i'' 
				if `"`lbl'"' == "" local lbl "``i''" 
				replace `thisuse' = `touse' & ``i'' < . 
			} 
							
			label def `whichlabel' `i' `"`lbl'"', add  
			count if `thisuse' 

			if "`by'`allobs'" != "" { 
				replace `n' = "`r(N)'" in `i' 
			} 
			
			if "`by'" != "" {
				mata : hs_mode("`varlist'", "`thisuse'")
			} 
			else mata: hs_mode("``i''", "`thisuse'") 

			replace `result' = string(r(hsmode), "`format'") in `i'  
			if "`generate'" != "" { 
				replace `generate' = r(hsmode) if `thisuse'
			}

			// leaves only last-calculated in memory 
			return scalar hsmode = r(hsmode)
			return scalar N = r(N) 
		}
	}	

	label val `which' `whichlabel' 
	tabdisp `which' in 1/`nv', cellvar(`n' `result') 
end 

mata : 

void hs_mode(string scalar varname, string scalar tousename) 
{ 
	real colvector y
	real scalar n, mode 

        y = st_data(., varname, tousename)    
	n = rows(y) 

	if (n <= 3) { 
		mode = report(y)
	}
	else { 
		real matrix diff 
		real scalar n2, i1, i2,	nties

		i1 = 1 
		_sort(y, 1) 

		while (n > 3) { 
			n2 = floor(n/2) 
			n1 = n - n2
			i2 = i1 + n2 
			diff = y[i2 .. (i2 + n1 - 1)] - y[i1 .. (i1 + n1 - 1)] 
			diff = diff, (i1 .. (i1 + n1 - 1))' 
			_sort(diff, (1,2))
			nties = colsum(diff[,1] :== diff[1,1]) 
			i1 = diff[ceil(nties / 2), 2] 
			n = n1 
		}

		mode = report(y[i1 .. (i1 + n1 - 1)]) 
	}

	st_numscalar("r(hsmode)", mode) 
}  	

real report(real colvector y) 
{ 
	if (rows(y) == 1) { 
		return(y[1])
	}
	else if (rows(y) == 2) {
		return((y[1] + y[2]) / 2) 
	}
	else if (rows(y) == 3) { 
		_sort(y, 1) 
		if ((y[2] - y[1]) < (y[3] - y[2])) { 
			return((y[1] + y[2]) / 2)
		}
		else if ((y[2] - y[1]) > (y[3] - y[2])) { 
			return((y[3] + y[2]) / 2)
		}
		else return(y[2]) 
	} 
}	

end

