*! 1.0.0 NJC 5 August 2007
program lmo_quantile, sort 
	version 9 
	syntax varname(numeric) [if] [in] [, nterms(numlist int >0 <9 sort) ///
	generate(str) * ] 

	quietly { 
		marksample touse
		count if `touse' 
		if r(N) == 0 error 2000

		if "`generate'" != "" { 
			confirm new var `generate' 
			local ng : word count `generate' 
		} 

		local N = r(N) 
		replace `touse' = -`touse' 
	
		sort `touse' `varlist' 

		if "`nterms'" == "" { 
			local nterms = 4 
			local maxnterms = 4
			local 1 = 4 
			local nt = 1 
		}
		else { 
			local nt : word count `nterms' 
			if `nt' != `ng' { 
				di as err "`nt' terms, `ng' new " ///
				plural(`ng', "variable")
				exit 498
			}

			tokenize `nterms' 
			local maxnterms = .   
			forval i = 1/`nt' { 
				local maxnterms = max(`maxnterms', ``i'')
			}

		        if inrange(`maxnterms', 5, 8) { 
				local lmax lmax(`maxnterms') 
			}
		}

		tempname coeff term 
		tempvar pp 
		gen `pp' = (_n - 0.5) / `N' if `touse' 
		mata: lmo_quantile_slp(`maxnterms', "`coeff'") 
		lmoments `varlist' if `touse', `lmax' 

		local menu solid dash dot dash_dot ///
		shortdash shortdash_dot longdash longdash_dot
		local nm = `nt' 
		local nl = 2 
		local N = min(`N', 200) 

		forval h = 1/`nt' { 
			tempvar qpred 
			gen double `qpred' = 0 

			forval i = 1/``h'' {
				scalar `term' = (2 * `i' - 1) * r(l_`i') 
				local exp 0 
				forval j = 1/`i' { 
					local exp ///
				`exp' + `coeff'[`i', `j'] * `pp'^(`j' - 1)
				}
				replace `qpred' = `qpred' + `term' * (`exp') 
			} 

		
			local style : word `nm' of `menu' 
			local call `call' mspline `qpred' `pp', ///
			lp(`style') bands(`N') || 
			local text = plural(``h'', "term") 
			local show `show' `nl' "``h'' `text'" 

			if "`generate'" != "" { 
				local togen : word `h' of `generate' 
				generate `togen' = `qpred' 
				label var `togen' "``h'' `text'" 
			} 

			local ++nl 
			local --nm 
		} 
	}

	local N = min(`N', 200) 
	local text = plural(`nt', "approximation") 

	qplot `varlist' if `touse', ///
	legend(order(`show') ring(0) pos(11) col(1)) ///
	xti(fraction of data) `options' ///
	addplot(`call')  
end 

/// shifted Legendre polynomial coefficients 
/// NJC 5 August 2007 

mata : 

real lmo_quantile_slp(real scalar r, string scalar matname) { 
	real matrix P 
	real scalar i, j 
	P = J(r, r, .)

	for(i = 0; i < r; i++) {
		for (j = 0; j < r; j++) {
			P[i + 1, j + 1] = (-1)^(i - j) * 
				comb(i, j) * comb(i + j, j)
		}
	}

	st_matrix(matname, editmissing(P, 0))  
} 

end 
